#ifndef BREUK_H
#define BREUK_H
#include <iostream>
#include "trace.h"

/** Een breuk wordt voorgesteld door een teller en een noemer, beiden
 * int.
 */
class Breuk {
public:
  /** Constructor.
   * @param t teller
   * @param n noemer
   * @warning abort als n=0
   */
  Breuk(int t =0, int n =1);

  /** Copy constructor 
   * @param b breuk waarvan een copie genomen wordt om deze breuk te
   *   initialiseren.
   */
  Breuk(const Breuk& b);

  /** Destructor. */
  ~Breuk();

  /** Tel b bij deze breuk op.
   * @param b breuk die opgeteld wordt bij deze breuk.
   * @return som van deze breuk en b
   */
  Breuk add(Breuk b);
  /** Vermenigvuldig deze breuk met b.
   * @param b breuk waarmee deze breuk vermenigvuldigd wordt.
   * @return produkt van deze breuk en b
   */
  Breuk multiply(Breuk b);

  /** Toon een breuk op een ostream. Een breuk wordt getoond
   * als "teller/noemer".
   * @param os ostream waarop de breuk getoond wordt.
   * @param b breuk die getoond wordt.
   * @return referentie naar os
   */
  friend std::ostream& operator<<(std::ostream& os, const Breuk& b);

  /** Totaal aantal bestaande Breuk objecten. */
  static int total;
private:
  /** Vb van een private member function.
   * Deze functie vereenvoudigt de breuk.
   */
  void simplify();

  int t_;
  int n_;
};

inline Breuk operator+(Breuk b1, Breuk b2) { return b1.add(b2); }
inline Breuk operator*(Breuk b1, Breuk b2) { return b1.multiply(b2); }
#endif
