#ifndef TRACE_H
#define TRACE_H

#include <iostream>
#include <string>

/** This class is useful for printing out a trace upon entry and exit
 * of a function.
 * 
 * Example usage:
 * @code
 * C::f(int i) {
 * Trace t(__PRETTY_FUNCTION__); // __PRETTY_FUNCTION__ is g++ extension
 * .. // rest of function
 * }
 * The constructor of t will print "Enter ( C::f" while
 * at the end of the function call (to f), the destructor
 * of t will print "Exit ) C::f". The parentheses are there for easy
 * matching using vi (use '%' key on one of them).
 */
class Trace {
public:
  /** Constructor.
   * A message will be printed to std::cerr.
   * @param s message to print
   */
  Trace(const std::string& s): message_(s) {
    std::cerr << "Enter ( " << s << std::endl;
    }
  /** Destructor.
   * A message will be printed to std::cerr.
   */
  ~Trace() {
    std::cerr << "Exit  ) " << message_ << std::endl;
    }
private:
  std::string message_;
};

#endif
